@@ -62,15 +62,18 @@ seek o = allowConcurrentOutput $ do
 		Batch
 			| updateOnly o ->
 				giveup "--update --batch is not supported"
-			| otherwise -> batchFiles gofile
+			| otherwise -> batchFilesMatching gofile
 		NoBatch -> do
 			l <- workTreeItems (addThese o)
 			let go a = a gofile l
 			unless (updateOnly o) $
 				go (withFilesNotInGit (not $ includeDotFiles o))
 			go withFilesMaybeModified
-			unlessM (versionSupportsUnlockedPointers <||> isDirect) $
-				go withFilesOldUnlocked
+			ifM versionSupportsUnlockedPointers
+				( go withUnmodifiedUnlockedPointers
+				, unlessM isDirect $
+					go withFilesOldUnlocked
+				)
 
 {- Pass file off to git-add. -}
 startSmall :: FilePath -> CommandStart
@@ -111,8 +114,7 @@ start file = do
 	addpresent key = ifM versionSupportsUnlockedPointers
 		( liftIO (catchMaybeIO $ getSymbolicLinkStatus file) >>= \case
 			Just s | isSymbolicLink s -> fixuplink key
-			_ -> ifM (sameInodeCache file =<< Database.Keys.getInodeCaches key)
-				( stop, add )
+			_ -> add
 		, ifM isDirect
 			( liftIO (catchMaybeIO $ getSymbolicLinkStatus file) >>= \case
 				Just s | isSymbolicLink s -> fixuplink key
