@@ -1,7 +1,9 @@
-{- git-annex general metadata storage log
+{- git-annex general metadata storage log and per-remote metadata storage log.
  -
  - A line of the log will look like "timestamp field [+-]value [...]"
  -
+ - (In the per-remote log, each field is prefixed with "uuid:")
+ -
  - Note that unset values are preserved. Consider this case:
  -
  - We have:
@@ -18,17 +20,15 @@
  - and so foo currently has no value.
  -
  -
- - Copyright 2014 Joey Hess <id@joeyh.name>
- -
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-
 module Logs.MetaData (
 	getCurrentMetaData,
+	getCurrentRemoteMetaData,
 	addMetaData,
-	addMetaData',
+	addRemoteMetaData,
+	addMetaDataClocked,
 	currentMetaData,
 	copyMetaData,
 ) where
@@ -40,24 +40,12 @@ import Annex.VectorClock
 import qualified Annex.Branch
 import qualified Annex
 import Logs
-import Logs.SingleValue
 import Logs.TimeStamp
+import Logs.MetaData.Pure
 
 import qualified Data.Set as S
 import qualified Data.Map as M
 
-instance SingleValueSerializable MetaData where
-	serialize = Types.MetaData.serialize
-	deserialize = Types.MetaData.deserialize
-
-getMetaDataLog :: Key -> Annex (Log MetaData)
-getMetaDataLog key = do
-	config <- Annex.getGitConfig
-	readLog $ metaDataLogFile config key
-
-logToCurrentMetaData :: [LogEntry MetaData] -> MetaData
-logToCurrentMetaData = currentMetaData . combineMetaData . map value
-
 {- Go through the log from oldest to newest, and combine it all
  - into a single MetaData representing the current state.
  -
@@ -65,8 +53,12 @@ logToCurrentMetaData = currentMetaData . combineMetaData . map value
  - currently set, based on timestamps in the log.
  -}
 getCurrentMetaData :: Key -> Annex MetaData
-getCurrentMetaData k = do
-	ls <- S.toAscList <$> getMetaDataLog k
+getCurrentMetaData = getCurrentMetaData' metaDataLogFile
+
+getCurrentMetaData' :: (GitConfig -> Key -> FilePath) -> Key -> Annex MetaData
+getCurrentMetaData' getlogfile k = do
+	config <- Annex.getGitConfig
+	ls <- S.toAscList <$> readLog (getlogfile config k)
 	let loggedmeta = logToCurrentMetaData ls
 	return $ currentMetaData $ unionMetaData loggedmeta
 		(lastchanged ls loggedmeta)
@@ -92,86 +84,46 @@ getCurrentMetaData k = do
 			Unknown -> 0
 	showts = formatPOSIXTime "%F@%H-%M-%S"
 
+getCurrentRemoteMetaData :: UUID -> Key -> Annex RemoteMetaData
+getCurrentRemoteMetaData u k = extractRemoteMetaData u <$>
+	getCurrentMetaData' remoteMetaDataLogFile k
+
 {- Adds in some metadata, which can override existing values, or unset
  - them, but otherwise leaves any existing metadata as-is. -}
 addMetaData :: Key -> MetaData -> Annex ()
-addMetaData k metadata = addMetaData' k metadata =<< liftIO currentVectorClock
+addMetaData = addMetaData' metaDataLogFile
+
+addMetaData' :: (GitConfig -> Key -> FilePath) -> Key -> MetaData -> Annex ()
+addMetaData' getlogfile k metadata = 
+	addMetaDataClocked' getlogfile k metadata =<< liftIO currentVectorClock
 
 {- Reusing the same VectorClock when making changes to the metadata
  - of multiple keys is a nice optimisation. The same metadata lines
  - will tend to be generated across the different log files, and so
  - git will be able to pack the data more efficiently. -}
-addMetaData' :: Key -> MetaData -> VectorClock -> Annex ()
-addMetaData' k d@(MetaData m) c
+addMetaDataClocked :: Key -> MetaData -> VectorClock -> Annex ()
+addMetaDataClocked = addMetaDataClocked' metaDataLogFile
+
+addMetaDataClocked' :: (GitConfig -> Key -> FilePath) -> Key -> MetaData -> VectorClock -> Annex ()
+addMetaDataClocked' getlogfile k d@(MetaData m) c
 	| d == emptyMetaData = noop
 	| otherwise = do
 		config <- Annex.getGitConfig
-		Annex.Branch.change (metaDataLogFile config k) $
+		Annex.Branch.change (getlogfile config k) $
 			showLog . simplifyLog 
 				. S.insert (LogEntry c metadata)
 				. parseLog
   where
 	metadata = MetaData $ M.filterWithKey (\f _ -> not (isLastChangedField f)) m
 
-{- Simplify a log, removing historical values that are no longer
- - needed. 
- -
- - This is not as simple as just making a single log line with the newest
- - state of all metadata. Consider this case:
- -
- - We have:
- -
- - 100 foo +x bar +y
- - 200 foo -x
- -
- - An unmerged remote has:
- -
- - 150 bar -y baz +w
- -
- - If what we have were simplified to "200 foo -x bar +y" then when the line
- - from the remote became available, it would be older than the simplified
- - line, and its change to bar would not take effect. That is wrong.
- -
- - Instead, simplify it to:
- -
- - 100 bar +y
- - 200 foo -x
- -
- - (Note that this ends up with the same number of lines as the
- - unsimplified version, so there's really no point in updating
- - the log to this version. Doing so would only add data to git,
- - with little benefit.)
- -
- - Now merging with the remote yields:
- -
- - 100 bar +y
- - 150 bar -y baz +w
- - 200 foo -x
- -
- - Simplifying again:
- -
- - 150 bar +z baz +w
- - 200 foo -x
- -}
-simplifyLog :: Log MetaData -> Log MetaData
-simplifyLog s = case sl of
-	(newest:rest) -> 
-		let sl' = go [newest] (value newest) rest
-		in if length sl' < length sl
-			then S.fromList sl'
-			else s
-	_ -> s
-  where
-	sl = S.toDescList s
+addRemoteMetaData :: Key -> RemoteMetaData -> Annex ()
+addRemoteMetaData k m = do
+	addMetaData' remoteMetaDataLogFile k (fromRemoteMetaData m)
 
-	go c _ [] = c
-	go c newer (l:ls)
-		| unique == emptyMetaData = go c newer ls
-		| otherwise = go (l { value = unique } : c)
-			(unionMetaData unique newer) ls
-	  where
-		older = value l
-		unique = older `differenceMetaData` newer
+getMetaDataLog :: Key -> Annex (Log MetaData)
+getMetaDataLog key = do
+	config <- Annex.getGitConfig
+	readLog $ metaDataLogFile config key
 
 {- Copies the metadata from the old key to the new key.
  -
@@ -195,3 +147,6 @@ copyMetaData oldkey newkey
 				Annex.Branch.change (metaDataLogFile config newkey) $
 					const $ showLog l
 				return True
+
+readLog :: FilePath -> Annex (Log MetaData)
+readLog = parseLog <$$> Annex.Branch.get
