@@ -66,8 +66,12 @@ trustMapLoad :: Annex TrustMap
 trustMapLoad = do
 	overrides <- Annex.getState Annex.forcetrust
 	l <- remoteList
-	-- Exports are never trusted, since they are not key/value stores.
-	exports <- filterM Types.Remote.isExportSupported l
+	-- Exports are not trusted, since they are not key/value stores.
+	-- This does not apply to appendonly exports, which are key/value
+	-- stores.
+	let untrustworthy r = pure (not (Types.Remote.appendonly r)) 
+		<&&> Types.Remote.isExportSupported r 
+	exports <- filterM untrustworthy l
 	let exportoverrides = M.fromList $
 		map (\r -> (Types.Remote.uuid r, UnTrusted)) exports
 	logged <- trustMapRaw
