@@ -83,6 +83,7 @@ data Message
 	| DATA Len -- followed by bytes of data
 	| VALIDITY Validity
 	| ERROR String
+	| ProtocolEOF
 	deriving (Show)
 
 instance Proto.Sendable Message where
@@ -108,6 +109,7 @@ instance Proto.Sendable Message where
 	formatMessage (VALIDITY Invalid) = ["INVALID"]
 	formatMessage (DATA len) = ["DATA", Proto.serialize len]
 	formatMessage (ERROR err) = ["ERROR", Proto.serialize err]
+	formatMessage (ProtocolEOF) = []
 
 instance Proto.Receivable Message where
 	parseCommand "AUTH" = Proto.parse2 AUTH
@@ -367,6 +369,8 @@ serverLoop :: (Message -> Proto (ServerHandler a)) -> Proto (Maybe a)
 serverLoop a = do
 	mcmd <- net receiveMessage
 	case mcmd of
+		-- Stop loop at EOF
+		Just ProtocolEOF -> return Nothing
 		-- When the client sends ERROR to the server, the server
 		-- gives up, since it's not clear what state the client
 		-- is in, and so not possible to recover.
