@@ -18,8 +18,9 @@ import qualified Aws.S3 as S3
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.ByteString.Lazy as L
-import qualified Data.ByteString as S
+import qualified Data.ByteString as BS
 import qualified Data.Map as M
+import qualified Data.Set as S
 import Data.Char
 import Network.Socket (HostName)
 import Network.HTTP.Conduit (Manager)
@@ -45,6 +46,8 @@ import qualified Remote.Helper.AWS as AWS
 import Creds
 import Annex.UUID
 import Logs.Web
+import Logs.MetaData
+import Types.MetaData
 import Utility.Metered
 import qualified Annex.Url as Url
 import Utility.DataUnits
@@ -52,6 +55,7 @@ import Utility.FileSystemEncoding
 import Annex.Content
 import Annex.Url (withUrlOptions)
 import Utility.Url (checkBoth, UrlOptions(..))
+import Utility.Env
 
 type BucketName = String
 type BucketObject = String
@@ -73,9 +77,9 @@ gen r u c gc = do
   where
 	new cst info = Just $ specialRemote c
 		(prepareS3Handle this $ store this info)
-		(prepareS3HandleMaybe this $ retrieve this info)
+		(prepareS3HandleMaybe this $ retrieve this c info)
 		(prepareS3Handle this $ remove info)
-		(prepareS3HandleMaybe this $ checkKey this info)
+		(prepareS3HandleMaybe this $ checkKey this c info)
 		this
 	  where
 		this = Remote
@@ -102,7 +106,7 @@ gen r u c gc = do
 					, removeExportDirectory = Nothing
 					, renameExport = renameExportS3 u info mh
 					}
-			, whereisKey = Just (getWebUrls info c)
+			, whereisKey = Just (getPublicWebUrls u info c)
 			, remoteFsck = Nothing
 			, repairRepo = Nothing
 			, config = c
@@ -110,6 +114,7 @@ gen r u c gc = do
 			, gitconfig = gc
 			, localpath = Nothing
 			, readonly = False
+			, appendonly = versioning info
 			, availability = GloballyAvailable
 			, remotetype = remote
 			, mkUnavailable = gen r u (M.insert "host" "!dne!" c) gc
@@ -186,13 +191,13 @@ prepareS3HandleMaybe r = resourcePrepare $ const $
 
 store :: Remote -> S3Info -> S3Handle -> Storer
 store _r info h = fileStorer $ \k f p -> do
-	storeHelper info h f (T.pack $ bucketObject info k) p
+	void $ storeHelper info h f (T.pack $ bucketObject info k) p
 	-- Store public URL to item in Internet Archive.
 	when (isIA info && not (isChunkKey k)) $
 		setUrlPresent webUUID k (iaPublicUrl info (bucketObject info k))
 	return True
 
-storeHelper :: S3Info -> S3Handle -> FilePath -> S3.Object -> MeterUpdate -> Annex ()
+storeHelper :: S3Info -> S3Handle -> FilePath -> S3.Object -> MeterUpdate -> Annex (Maybe S3VersionID)
 storeHelper info h f object p = case partSize info of
 	Just partsz | partsz > 0 -> do
 		fsz <- liftIO $ getFileSize f
@@ -203,9 +208,10 @@ storeHelper info h f object p = case partSize info of
   where
 	singlepartupload = do
 		rbody <- liftIO $ httpBodyStorer f p
-		void $ sendS3Handle h $ putObject info object rbody
+		r <- sendS3Handle h $ putObject info object rbody
+		return (mkS3VersionID object (S3.porVersionId r))
 	multipartupload fsz partsz = do
-#if MIN_VERSION_aws(0,10,6)
+#if MIN_VERSION_aws(0,16,0)
 		let startreq = (S3.postInitiateMultipartUpload (bucket info) object)
 				{ S3.imuStorageClass = Just (storageClass info)
 				, S3.imuMetadata = metaHeaders info
@@ -240,8 +246,9 @@ storeHelper info h f object p = case partSize info of
 						sendparts (offsetMeterUpdate meter (toBytesProcessed sz)) (etag:etags) (partnum + 1)
 			sendparts p [] 1
 
-		void $ sendS3Handle h $ S3.postCompleteMultipartUpload
+		r <- sendS3Handle h $ S3.postCompleteMultipartUpload
 			(bucket info) object uploadid (zip [1..] etags)
+		return (mkS3VersionID object (S3.cmurVersionId r))
 #else
 		warning $ "Cannot do multipart upload (partsize " ++ show partsz ++ ") of large file (" ++ show fsz ++ "); built with too old a version of the aws library."
 		singlepartupload
@@ -250,20 +257,24 @@ storeHelper info h f object p = case partSize info of
 {- Implemented as a fileRetriever, that uses conduit to stream the chunks
  - out to the file. Would be better to implement a byteRetriever, but
  - that is difficult. -}
-retrieve :: Remote -> S3Info -> Maybe S3Handle -> Retriever
-retrieve _ info (Just h) = fileRetriever $ \f k p ->
-	retrieveHelper info h (T.pack $ bucketObject info k) f p
-retrieve r info Nothing = case getpublicurl info of
-	Nothing -> \_ _ _ -> do
-		needS3Creds (uuid r)
-		return False
-	Just geturl -> fileRetriever $ \f k p ->
-		unlessM (downloadUrl k p [geturl $ bucketObject info k] f) $
+retrieve :: Remote -> RemoteConfig -> S3Info -> Maybe S3Handle -> Retriever
+retrieve r _ info (Just h) = fileRetriever $ \f k p -> do
+	loc <- eitherS3VersionID info (uuid r) k (T.pack $ bucketObject info k)
+	retrieveHelper info h loc f p
+retrieve r c info Nothing = fileRetriever $ \f k p ->
+	getPublicWebUrls (uuid r) info c k >>= \case
+		[] -> do
+			needS3Creds (uuid r)
+			giveup "No S3 credentials configured"
+		us -> unlessM (downloadUrl k p us f) $
 			giveup "failed to download content"
 
-retrieveHelper :: S3Info -> S3Handle -> S3.Object -> FilePath -> MeterUpdate -> Annex ()
-retrieveHelper info h object f p = liftIO $ runResourceT $ do
-	let req = S3.getObject (bucket info) object
+retrieveHelper :: S3Info -> S3Handle -> (Either S3.Object S3VersionID) -> FilePath -> MeterUpdate -> Annex ()
+retrieveHelper info h loc f p = liftIO $ runResourceT $ do
+	let req = case loc of
+		Left o -> S3.getObject (bucket info) o
+		Right (S3VersionID o vid) -> (S3.getObject (bucket info) o)
+			{ S3.goVersionId = Just (T.pack vid) }
 	S3.GetObjectResponse { S3.gorResponse = rsp } <- sendS3Handle' h req
 	Url.sinkResponseFile p zeroBytesProcessed f WriteMode rsp
 
@@ -279,21 +290,24 @@ remove info h k = do
 		S3.DeleteObject (T.pack $ bucketObject info k) (bucket info)
 	return $ either (const False) (const True) res
 
-checkKey :: Remote -> S3Info -> Maybe S3Handle -> CheckPresent
-checkKey r info Nothing k = case getpublicurl info of
-	Nothing -> do
-		needS3Creds (uuid r)
-		giveup "No S3 credentials configured"
-	Just geturl -> do
-		showChecking r
-		withUrlOptions $ liftIO . 
-			checkBoth (geturl $ bucketObject info k) (keySize k)
-checkKey r info (Just h) k = do
+checkKey :: Remote -> RemoteConfig -> S3Info -> Maybe S3Handle -> CheckPresent
+checkKey r _ info (Just h) k = do
 	showChecking r
-	checkKeyHelper info h (T.pack $ bucketObject info k)
+	loc <- eitherS3VersionID info (uuid r) k (T.pack $ bucketObject info k)
+	checkKeyHelper info h loc
+checkKey r c info Nothing k = 
+	getPublicWebUrls (uuid r) info c k >>= \case
+		[] -> do
+			needS3Creds (uuid r)
+			giveup "No S3 credentials configured"
+		us -> do
+			showChecking r
+			let check u = withUrlOptions $ 
+				liftIO . checkBoth u (keySize k)
+			anyM check us
 
-checkKeyHelper :: S3Info -> S3Handle -> S3.Object -> Annex Bool
-checkKeyHelper info h object = do
+checkKeyHelper :: S3Info -> S3Handle -> (Either S3.Object S3VersionID) -> Annex Bool
+checkKeyHelper info h loc = do
 #if MIN_VERSION_aws(0,10,0)
 	rsp <- go
 	return (isJust $ S3.horMetadata rsp)
@@ -303,7 +317,11 @@ checkKeyHelper info h object = do
 		return True
 #endif
   where
-	go = sendS3Handle h $ S3.headObject (bucket info) object
+	go = sendS3Handle h req
+	req = case loc of
+		Left o -> S3.headObject (bucket info) o
+		Right (S3VersionID o vid) -> (S3.headObject (bucket info) o)
+			{ S3.hoVersionId = Just (T.pack vid) }
 
 #if ! MIN_VERSION_aws(0,10,0)
 	{- Catch exception headObject returns when an object is not present
@@ -319,11 +337,13 @@ checkKeyHelper info h object = do
 #endif
 
 storeExportS3 :: UUID -> S3Info -> Maybe S3Handle -> FilePath -> Key -> ExportLocation -> MeterUpdate -> Annex Bool
-storeExportS3 _u info (Just h) f _k loc p = 
+storeExportS3 u info (Just h) f k loc p = 
 	catchNonAsync go (\e -> warning (show e) >> return False)
   where
 	go = do
-		storeHelper info h f (T.pack $ bucketExportLocation info loc) p
+		let o = T.pack $ bucketExportLocation info loc
+		storeHelper info h f o p
+			>>= setS3VersionID info u k
 		return True
 storeExportS3 u _ Nothing _ _ _ _ = do
 	needS3Creds u
@@ -335,9 +355,9 @@ retrieveExportS3 u info mh _k loc f p =
   where
 	go = case mh of
 		Just h -> do
-			retrieveHelper info h (T.pack exporturl) f p
+			retrieveHelper info h (Left (T.pack exporturl)) f p
 			return True
-		Nothing -> case getpublicurl info of
+		Nothing -> case getPublicUrlMaker info of
 			Nothing -> do
 				needS3Creds u
 				return False
@@ -359,8 +379,8 @@ removeExportS3 u _ Nothing _ _ = do
 
 checkPresentExportS3 :: UUID -> S3Info -> Maybe S3Handle -> Key -> ExportLocation -> Annex Bool
 checkPresentExportS3 _u info (Just h) _k loc =
-	checkKeyHelper info h (T.pack $ bucketExportLocation info loc)
-checkPresentExportS3 u info Nothing k loc = case getpublicurl info of
+	checkKeyHelper info h (Left (T.pack $ bucketExportLocation info loc))
+checkPresentExportS3 u info Nothing k loc = case getPublicUrlMaker info of
 	Nothing -> do
 		needS3Creds u
 		giveup "No S3 credentials configured"
@@ -536,14 +556,14 @@ s3Configuration c = cfg
 	proto
 		| port == 443 = AWS.HTTPS
 		| otherwise = AWS.HTTP
-	host = fromJust $ M.lookup "host" c
+	h = fromJust $ M.lookup "host" c
 	datacenter = fromJust $ M.lookup "datacenter" c
 	-- When the default S3 host is configured, connect directly to
 	-- the S3 endpoint for the configured datacenter.
 	-- When another host is configured, it's used as-is.
 	endpoint
-		| host == AWS.s3DefaultHost = AWS.s3HostName $ T.pack datacenter
-		| otherwise = T.encodeUtf8 $ T.pack host
+		| h == AWS.s3DefaultHost = AWS.s3HostName $ T.pack datacenter
+		| otherwise = T.encodeUtf8 $ T.pack h
 	port = let s = fromJust $ M.lookup "port" c in
 		case reads s of
 		[(p, _)] -> p
@@ -558,8 +578,10 @@ data S3Info = S3Info
 	, metaHeaders :: [(T.Text, T.Text)]
 	, partSize :: Maybe Integer
 	, isIA :: Bool
+	, versioning :: Bool
 	, public :: Bool
-	, getpublicurl :: Maybe (BucketObject -> URLString)
+	, publicurl :: Maybe URLString
+	, host :: Maybe String
 	}
 
 extractS3Info :: RemoteConfig -> Annex S3Info
@@ -568,7 +590,7 @@ extractS3Info c = do
 		(giveup "S3 bucket not configured")
 		(return . T.pack)
 		(getBucketName c)
-	let info = S3Info
+	return $ S3Info
 		{ bucket = b
 		, storageClass = getStorageClass c
 		, bucketObject = getBucketObject c
@@ -576,20 +598,15 @@ extractS3Info c = do
 		, metaHeaders = getMetaHeaders c
 		, partSize = getPartSize c
 		, isIA = configIA c
-		, public = case M.lookup "public" c of
-			Just "yes" -> True
-			_ -> False
-		, getpublicurl = case M.lookup "publicurl" c of
-			Just u -> Just $ \p -> genericPublicUrl p u
-			Nothing -> case M.lookup "host" c of
-				Just h
-					| h == AWS.s3DefaultHost ->
-						Just (awsPublicUrl info)
-					| isIAHost h ->
-						Just (iaPublicUrl info)
-				_ -> Nothing
+		, versioning = boolcfg "versioning"
+		, public = boolcfg "public"
+		, publicurl = M.lookup "publicurl" c
+		, host = M.lookup "host" c
 		}
-	return info
+  where
+	boolcfg k = case M.lookup k c of
+		Just "yes" -> True
+		_ -> False
 
 putObject :: S3Info -> T.Text -> RequestBody -> S3.PutObject
 putObject info file rbody = (S3.putObject (bucket info) file rbody)
@@ -665,22 +682,24 @@ iaItemUrl :: BucketName -> URLString
 iaItemUrl b = "http://archive.org/details/" ++ b
 
 iaPublicUrl :: S3Info -> BucketObject -> URLString
-iaPublicUrl info p = genericPublicUrl p $
+iaPublicUrl info = genericPublicUrl $
 	"http://archive.org/download/" ++ T.unpack (bucket info) ++ "/" 
 
 awsPublicUrl :: S3Info -> BucketObject -> URLString
-awsPublicUrl info p = genericPublicUrl p $ 
+awsPublicUrl info = genericPublicUrl $ 
 	"https://" ++ T.unpack (bucket info) ++ ".s3.amazonaws.com/" 
 
-genericPublicUrl :: BucketObject -> URLString -> URLString
-genericPublicUrl p baseurl = baseurl ++ p
+genericPublicUrl :: URLString -> BucketObject -> URLString
+genericPublicUrl baseurl p = baseurl ++ p
 
 genCredentials :: CredPair -> IO AWS.Credentials
 genCredentials (keyid, secret) = AWS.Credentials
-	<$> pure (T.encodeUtf8 (T.pack keyid))
-	<*> pure (T.encodeUtf8 (T.pack secret))
+	<$> pure (tobs keyid)
+	<*> pure (tobs secret)
 	<*> newIORef []
-	<*> pure Nothing
+	<*> (fmap tobs <$> getEnv "AWS_SESSION_TOKEN")
+  where
+	tobs = T.encodeUtf8 . T.pack
 
 mkLocationConstraint :: AWS.Region -> S3.LocationConstraint
 mkLocationConstraint "US" = S3.locationUsClassic
@@ -698,7 +717,7 @@ debugMapper level t = forward "S3" (T.unpack t)
 s3Info :: RemoteConfig -> S3Info -> [(String, String)]
 s3Info c info = catMaybes
 	[ Just ("bucket", fromMaybe "unknown" (getBucketName c))
-	, Just ("endpoint", w82s (S.unpack (S3.s3Endpoint s3c)))
+	, Just ("endpoint", w82s (BS.unpack (S3.s3Endpoint s3c)))
 	, Just ("port", show (S3.s3Port s3c))
 	, Just ("storage class", showstorageclass (getStorageClass c))
 	, if configIA c
@@ -706,6 +725,7 @@ s3Info c info = catMaybes
 		else Nothing
 	, Just ("partsize", maybe "unlimited" (roughSize storageUnits False) (getPartSize c))
 	, Just ("public", if public info then "yes" else "no")
+	, Just ("versioning", if versioning info then "yes" else "no")
 	]
   where
 	s3c = s3Configuration c
@@ -714,10 +734,102 @@ s3Info c info = catMaybes
 #endif
 	showstorageclass sc = show sc
 
-getWebUrls :: S3Info -> RemoteConfig -> Key -> Annex [URLString]
-getWebUrls info c k
-	| exportTree c = return []
-	| otherwise = case (public info, getpublicurl info) of
-		(True, Just geturl) -> return [geturl $ bucketObject info k]
-		_ -> return []
+getPublicWebUrls :: UUID -> S3Info -> RemoteConfig -> Key -> Annex [URLString]
+getPublicWebUrls u info c k
+	| not (public info) = return []
+	| exportTree c = if versioning info
+		then case publicurl info of
+			Just url -> getS3VersionIDPublicUrls (const $ genericPublicUrl url) info u k
+			Nothing -> case host info of
+				Just h | h == AWS.s3DefaultHost ->
+					getS3VersionIDPublicUrls awsPublicUrl info u k
+				_ -> return []
+		else return []
+	| otherwise = case getPublicUrlMaker info of
+		Just geturl -> return [geturl $ bucketObject info k]
+		Nothing -> return []
 
+getPublicUrlMaker :: S3Info -> Maybe (BucketObject -> URLString)
+getPublicUrlMaker info = case publicurl info of
+	Just url -> Just (genericPublicUrl url)
+	Nothing -> case host info of
+		Just h
+			| h == AWS.s3DefaultHost ->
+				Just (awsPublicUrl info)
+			| isIAHost h ->
+				Just (iaPublicUrl info)
+		_ -> Nothing
+
+
+data S3VersionID = S3VersionID S3.Object String
+	deriving (Show)
+
+-- smart constructor
+mkS3VersionID :: S3.Object -> Maybe T.Text -> Maybe S3VersionID
+mkS3VersionID o = mkS3VersionID' o . fmap T.unpack
+
+mkS3VersionID' :: S3.Object -> Maybe String -> Maybe S3VersionID
+mkS3VersionID' o (Just s)
+	| null s = Nothing
+	-- AWS documentation says a version ID is at most 1024 bytes long.
+	-- Since they are stored in the git-annex branch, prevent them from
+	-- being very much larger than that.
+	| length s < 2048 = Just (S3VersionID o s)
+	| otherwise = Nothing
+mkS3VersionID' _ Nothing = Nothing
+
+-- Format for storage in per-remote metadata.
+--
+-- A S3 version ID is "url ready" so does not contain '#' and so we'll use
+-- that to separate it from the object id. (Could use a space, but spaces
+-- in metadata values lead to an inefficient encoding.)
+formatS3VersionID :: S3VersionID -> String
+formatS3VersionID (S3VersionID o v) = v ++ '#' : T.unpack o
+
+-- Parse from value stored in per-remote metadata.
+parseS3VersionID :: String -> Maybe S3VersionID
+parseS3VersionID s = 
+	let (v, o) = separate (== '#') s
+	in mkS3VersionID' (T.pack o) (Just v)
+
+setS3VersionID :: S3Info -> UUID -> Key -> Maybe S3VersionID -> Annex ()
+setS3VersionID info u k vid
+	| versioning info = maybe noop (setS3VersionID' u k) vid
+	| otherwise = noop
+
+setS3VersionID' :: UUID -> Key -> S3VersionID -> Annex ()
+setS3VersionID' u k vid = addRemoteMetaData k $
+	RemoteMetaData u (updateMetaData s3VersionField v emptyMetaData)
+  where
+	v = mkMetaValue (CurrentlySet True) (formatS3VersionID vid)
+
+getS3VersionID :: UUID -> Key -> Annex [S3VersionID]
+getS3VersionID u k = do
+	(RemoteMetaData _ m) <- getCurrentRemoteMetaData u k
+	return $ mapMaybe parseS3VersionID $ map unwrap $ S.toList $
+		metaDataValues s3VersionField m
+  where
+	unwrap (MetaValue _ v) = v
+
+s3VersionField :: MetaField
+s3VersionField = mkMetaFieldUnchecked "V"
+
+eitherS3VersionID :: S3Info -> UUID -> Key -> S3.Object -> Annex (Either S3.Object S3VersionID)
+eitherS3VersionID info u k fallback
+	| versioning info = getS3VersionID u k >>= return . \case
+		[] -> Left fallback
+		-- It's possible for a key to be stored multiple timees in
+		-- a bucket with different version IDs; only use one of them.
+		(v:_) -> Right v
+	| otherwise = return (Left fallback)
+
+s3VersionIDPublicUrl :: (S3Info -> BucketObject -> URLString) -> S3Info -> S3VersionID -> URLString
+s3VersionIDPublicUrl mk info (S3VersionID obj vid) = mk info $ concat
+	[ T.unpack obj
+	, "?versionId="
+	, vid -- version ID is "url ready" so no escaping needed
+	]
+
+getS3VersionIDPublicUrls :: (S3Info -> BucketObject -> URLString) -> S3Info -> UUID -> Key -> Annex [URLString]
+getS3VersionIDPublicUrls mk info u k =
+	map (s3VersionIDPublicUrl mk info) <$> getS3VersionID u k
